//
//  MATCMapViewController.m
//  MATCMap
//
//  Created by Brad Larson on 3/1/2010.
//

#import "MATCMapViewController.h"

@implementation MATCMapViewController

#pragma mark -
#pragma mark Initialization and teardown

- (void)viewDidLoad 
{
	mapView.delegate = self;
	mapView.mapType = MKMapTypeStandard; // Can also be MKMapTypeSatellite or MKMapTypeHybrid
	mapView.zoomEnabled = YES;
	mapView.scrollEnabled = YES;
	mapView.showsUserLocation = YES;
	
	matcLocationAnnotation = [[MATCAnnotation alloc] init];
	CLLocationCoordinate2D matcCoordinate;
	matcCoordinate.latitude = 43.1209332;
	matcCoordinate.longitude = -89.3267498;
	matcLocationAnnotation.coordinate = matcCoordinate;
	matcLocationAnnotation.title = @"MATC";
	matcLocationAnnotation.subtitle = @"3550 Anderson Street, Madison, WI 53704";
//  userLocationVisible  property - readonly
//  userLocation  property
	
	[mapView addAnnotation:matcLocationAnnotation];
	
	// Start observing changes in the user's location, as reported by MKMapView
	[mapView.userLocation addObserver:self forKeyPath:@"location" options:0 context:NULL];
	
    [super viewDidLoad];
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc 
{
	[matcLocationAnnotation release];
    [super dealloc];
}

#pragma mark -
#pragma mark MKMapViewDelegate methods

- (void)mapView:(MKMapView *)mapView annotationView:(MKAnnotationView *)view calloutAccessoryControlTapped:(UIControl *)control
{
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Detail disclosure tapped" message:@"You tapped on the MATC location disclosure view."
												   delegate:self cancelButtonTitle:@"OK" otherButtonTitles: nil];
	[alert show];
	[alert release];
}

- (void)mapView:(MKMapView *)mapView didAddAnnotationViews:(NSArray *)views
{
	// Handle the addition of annotations to the map
}

- (void)mapView:(MKMapView *)mapView regionDidChangeAnimated:(BOOL)animated
{
	// Handle any changes after the region has shifted or zoomed
}

- (void)mapView:(MKMapView *)mapView regionWillChangeAnimated:(BOOL)animated
{
	// Handle any changes before the region has shifted or zoomed
}

// Change the color of the pin to green and add a disclosure arrow
- (MKAnnotationView *)mapView:(MKMapView *)theMapView viewForAnnotation:(id <MKAnnotation>)annotation
{
	MKPinAnnotationView *view = nil;
	if (annotation != theMapView.userLocation)
	{
		view = (MKPinAnnotationView *)[theMapView dequeueReusableAnnotationViewWithIdentifier:@"MATCLocationPinView"];
		if (view == nil)
		{
			view = [[[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"MATCLocationPinView"] autorelease];
			view.rightCalloutAccessoryView = [UIButton buttonWithType:UIButtonTypeDetailDisclosure];
			[view setPinColor:MKPinAnnotationColorGreen];
			[view setCanShowCallout:YES];
			[view setAnimatesDrop:YES];
		}
	}
	
	return view;
}

- (void)mapViewDidFailLoadingMap:(MKMapView *)mapView withError:(NSError *)error
{
	NSLog(@"Map view loading failed");
}

- (void)mapViewDidFinishLoadingMap:(MKMapView *)mapView
{
	[loadingActivityIndicator stopAnimating];
	loadingLabel.hidden = YES;
}

- (void)mapViewWillStartLoadingMap:(MKMapView *)mapView
{
	[loadingActivityIndicator startAnimating];
	loadingLabel.hidden = NO;
}

#pragma mark -
#pragma mark KVO methods

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context 
{
    static CFAbsoluteTime lastTime = 0.0;
    CFAbsoluteTime now = CFAbsoluteTimeGetCurrent();
    if((now - lastTime) > 15.0) 
	{ // 15 seconds
        lastTime = now;
		
		CLLocationCoordinate2D currentCoordinate = mapView.userLocation.location.coordinate;
		CLLocationCoordinate2D matcCoordinate = matcLocationAnnotation.coordinate;

		MKCoordinateRegion regionToDisplay;
		regionToDisplay.span.latitudeDelta = abs(matcCoordinate.latitude - currentCoordinate.latitude);
		regionToDisplay.span.longitudeDelta = abs(matcCoordinate.longitude - currentCoordinate.longitude);
		regionToDisplay.center.latitude = (matcCoordinate.latitude + currentCoordinate.latitude) / 2.0;
		regionToDisplay.center.longitude = (matcCoordinate.longitude + currentCoordinate.longitude) / 2.0;
		
		[mapView setRegion:regionToDisplay animated:YES];
    }
}

@end
